@@ -24,13 +24,14 @@
 //! is commented with careful justifications to *prove* correctness (at least it is a try).
 
 mod head;
+mod sort;
 
 pub use head::cursor::{Cursor, DoubleCursor};
 
 use {
     crate::head::{Iter, IterMut, ListHead},
+    core::{iter::FromIterator, ops::Not, ptr},
     either::Either,
-    std::{iter::FromIterator, ops::Not, ptr},
 };
 
 /// Create a list with provided elements.
@@ -73,6 +74,25 @@ pub struct CircularList<T> {
     // Invariant (2): It is updated each time an element is added to the list or removed from it.
     length: usize,
 }
+
+// private functions
+impl<T> CircularList<T> {
+    /// Creates a new `ListHead` with value `val` and places it after the element pointed by
+    /// `element`.
+    ///
+    /// # Safety
+    /// The caller must assert that `element` is a valid pointer to a `ListHead` that is a
+    /// member of `self`.
+    unsafe fn insert_after(&mut self, val: T, element: *mut ListHead<T>) {
+        let new = Box::leak(ListHead::<T>::new(val));
+
+        (*element).add_after(new);
+
+        // Preserving invariant (2)
+        self.length += 1;
+    }
+}
+
 impl<T> CircularList<T> {
     /// Gets the size of the list.
     pub fn len(&self) -> usize {
@@ -236,6 +256,64 @@ impl<T> CircularList<T> {
         other.length = 0;
     }
 
+    /// Assembles this list with another by keeping the elements sorted.
+    /// It is assumed that the 2 lists are sorted.
+    pub fn merge(&mut self, mut other: Self)
+    where
+        T: core::cmp::PartialOrd,
+    {
+        if self.is_empty() {
+            self.append(other);
+        } else if other.is_empty().not() {
+            let mut s = self.head as *mut ListHead<T>;
+            let mut s_head = s;
+            let mut o = other.head as *mut ListHead<T>;
+            unsafe {
+                let mut s_next = (*s).next() as *mut ListHead<T>;
+                let mut o_next = (*o).next() as *mut ListHead<T>;
+                let o_end = (*o).prev() as *mut ListHead<T>;
+                if *o < *s {
+                    self.head = o;
+                    s_head = o;
+                }
+                let mut reached_end_of_s = false;
+                loop {
+                    if reached_end_of_s {
+                        ListHead::add_list(o, s_head);
+                        break;
+                    } else if *o < *s {
+                        ListHead::move_entry(o, (*s).prev() as *mut _, s);
+                        if o == o_end {
+                            // We reached the last element of `other`
+                            break;
+                        }
+                        o = o_next;
+                        o_next = (*o).next() as *mut ListHead<T>;
+                    } else {
+                        s = s_next;
+                        s_next = (*s).next() as *mut ListHead<T>;
+                        reached_end_of_s = s == s_head;
+                    }
+                }
+            }
+
+            // Preserving invariant (1)
+            self.length += other.length;
+
+            // `other` must be in valid state when being dropped.
+            other.head = ptr::null();
+            other.length = 0;
+        }
+    }
+
+    /// Sorts the list.
+    pub fn sort(&mut self)
+    where
+        T: core::cmp::PartialOrd,
+    {
+        sort::sort::<T, sort::MergeSort<T>>(self)
+    }
+
     /// Moves the head `count` steps to the left.
     ///
     /// # Example
@@ -290,9 +368,9 @@ impl<T> CircularList<T> {
     /// returned iterator is also empty.
     pub fn iter_forever(&self) -> impl Iterator<Item = &T> {
         if self.is_empty() {
-            Either::Left(std::iter::empty())
+            Either::Left(core::iter::empty())
         } else {
-            Either::Right(Iter::new(self.head))
+            Either::Right(Iter::new(self))
         }
     }
 
@@ -305,9 +383,9 @@ impl<T> CircularList<T> {
     /// empty if the list is empty.
     pub fn iter_mut_forever(&mut self) -> impl Iterator<Item = &mut T> {
         if self.is_empty() {
-            Either::Left(std::iter::empty())
+            Either::Left(core::iter::empty())
         } else {
-            Either::Right(IterMut::new(self.head as *mut _))
+            Either::Right(IterMut::new(self))
         }
     }
 
@@ -364,8 +442,8 @@ impl<T> Drop for CircularList<T> {
     }
 }
 
-impl<T: std::fmt::Debug> std::fmt::Debug for CircularList<T> {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+impl<T: core::fmt::Debug> core::fmt::Debug for CircularList<T> {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         f.debug_list().entries(self.iter()).finish()
     }
 }
@@ -538,101 +616,24 @@ mod tests {
     }
 
     #[test]
-    fn cursor_empty_list() {
-        assert_eq!(CircularList::<()>::default().cursor(), None)
-    }
-
-    #[test]
-    fn cursor() {
-        let list = list![1, 2, 3, 4, 5];
-        let mut c1 = list
-            .cursor()
-            .expect("A cursor should always be available on a non empty list");
-
-        assert_eq!(c1.value(), &1);
-
-        c1.move_prev();
-        assert_eq!(c1.value(), &5);
-
-        for _ in 0..5 {
-            c1.move_next();
+    fn merge() {
+        for (mut a, b, c) in [
+            (list![1, 2, 3], list![], vec![1, 2, 3]),
+            (list![], list![1, 2, 3], vec![1, 2, 3]),
+            (list![1], list![0], vec![0, 1]),
+            (list![1], list![0], vec![0, 1]),
+            (list![1, 2, 3], list![4, 5, 6], vec![1, 2, 3, 4, 5, 6]),
+            (list![1, 3, 5], list![2, 4, 6], vec![1, 2, 3, 4, 5, 6]),
+            (list![1, 3, 5], list![0, 2, 4, 6], vec![0, 1, 2, 3, 4, 5, 6]),
+            (list![1, 3, 5], list![2, 4], vec![1, 2, 3, 4, 5]),
+            (list![1, 3, 5], list![2, 2, 4, 4], vec![1, 2, 2, 3, 4, 4, 5]),
+            (list![1, 3], list![0, 0, 4, 8], vec![0, 0, 1, 3, 4, 8]),
+            (list![7, 8, 9], list![1, 2, 8], vec![1, 2, 7, 8, 8, 9]),
+            (list![5, 5, 6], list![5, 8, 9], vec![5, 5, 5, 6, 8, 9]),
+            (list![1, 2, 3], list![1, 2, 3], vec![1, 1, 2, 2, 3, 3]),
+        ] {
+            a.merge(b);
+            assert_eq!(a.into_iter().collect::<Vec<i32>>(), c);
         }
-        assert_eq!(c1.value(), &5);
-
-        c1.move_next();
-        c1.move_next();
-        assert_eq!(c1.value(), &2);
-    }
-
-    #[test]
-    fn double_cursor_empty_list() {
-        assert!(matches!(
-            CircularList::<()>::default().double_cursor(),
-            None
-        ))
-    }
-
-    #[test]
-    fn double_cursor_swap() {
-        let mut list = list![1, 2, 3, 4, 5];
-        let mut dc = list
-            .double_cursor()
-            .expect("A cursor should always be available on a non empty list");
-
-        dc.move_next_b();
-        dc.swap();
-        assert_eq!(list.into_iter().collect::<Vec<i32>>(), &[2, 1, 3, 4, 5]);
-
-        let mut list = list![0];
-        let mut dc = list.double_cursor().unwrap();
-        dc.swap();
-        assert_eq!(list.into_iter().collect::<Vec<i32>>(), &[0]);
-    }
-
-    #[test]
-    fn double_cursor_move() {
-        let mut list = list![1, 2, 3, 4, 5];
-        let mut dc = list
-            .double_cursor()
-            .expect("A cursor should always be available on a non empty list");
-
-        dc.move_next_b();
-        dc.insert_a_after_b();
-        // This function is idempotent
-        dc.insert_a_after_b();
-        assert_eq!(list.into_iter().collect::<Vec<i32>>(), &[2, 1, 3, 4, 5]);
-    }
-
-    #[test]
-    fn double_cursor_sort() {
-        let mut list = list![3, 1, 8, 21, 5, 9, 12, 5, 2, 6, 6, 6, 13, 2, 17];
-        let len = list.len();
-        let mut dc = list
-            .double_cursor()
-            .expect("A cursor should always be available on a non empty list");
-
-        let mut min = *dc.value_a();
-        for i in 1..len {
-            dc.set_b_to_a();
-            dc.push_a();
-            for _ in i..len {
-                dc.move_next_a();
-                let val = *dc.value_a();
-                if val < min {
-                    min = val;
-                    dc.set_b_to_a();
-                }
-            }
-            dc.pop_a();
-            dc.insert_b_before_a();
-            if dc.a_is_b() {
-                dc.move_next_a();
-            }
-            min = *dc.value_a();
-        }
-        assert_eq!(
-            list.into_iter().collect::<Vec<i32>>(),
-            &[1, 2, 2, 3, 5, 5, 6, 6, 6, 8, 9, 12, 13, 17, 21]
-        );
     }
 }
